@@ -1,35 +1,35 @@
 /*
- * Copyright © 2010 Stéphane Raimbault <stephane.raimbault@gmail.com>
+ * Copyright © 2010-2011 Stéphane Raimbault <stephane.raimbault@gmail.com>
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _MODBUS_PRIVATE_H_
 #define _MODBUS_PRIVATE_H_
 
-#ifdef HAVE_SYS_TIME_H
+#ifndef _MSC_VER
+# include <stdint.h>
 # include <sys/time.h>
+#else
+# include "stdint.h"
+# include <time.h>
 #endif
 #include <sys/types.h>
-#include <stdint.h>
-
 #include <config.h>
-#include "modbus.h"
 
-#ifndef ssize_t
-# define ssize_t size_t
-#endif
+#include "modbus.h"
 
 MODBUS_BEGIN_DECLS
 
@@ -40,15 +40,15 @@ MODBUS_BEGIN_DECLS
  * - HEADER_LENGTH_TCP (7) + function (1) + address (2) + number (2)
  * - HEADER_LENGTH_RTU (1) + function (1) + address (2) + number (2) + CRC (2)
  */
-#define _MIN_REQ_LENGTH           12
+#define _MIN_REQ_LENGTH 12
 
-#define _REPORT_SLAVE_ID_LENGTH   75
+#define _REPORT_SLAVE_ID 180
 
-#define _MODBUS_EXCEPTION_RSP_LENGTH  5
+#define _MODBUS_EXCEPTION_RSP_LENGTH 5
 
-/* Time out between trames in microsecond */
-#define _TIME_OUT_BEGIN_OF_TRAME    500000
-#define _TIME_OUT_END_OF_TRAME      500000
+/* Timeouts in microsecond (0.5 s) */
+#define _RESPONSE_TIMEOUT    500000
+#define _BYTE_TIMEOUT        500000
 
 /* Function codes */
 #define _FC_READ_COILS                0x01
@@ -61,7 +61,7 @@ MODBUS_BEGIN_DECLS
 #define _FC_WRITE_MULTIPLE_COILS      0x0F
 #define _FC_WRITE_MULTIPLE_REGISTERS  0x10
 #define _FC_REPORT_SLAVE_ID           0x11
-#define _FC_READ_AND_WRITE_REGISTERS  0x17
+#define _FC_WRITE_AND_READ_REGISTERS  0x17
 
 typedef enum {
     _MODBUS_BACKEND_TYPE_RTU=0,
@@ -91,6 +91,8 @@ typedef struct _modbus_backend {
     ssize_t (*recv) (modbus_t *ctx, uint8_t *rsp, int rsp_length);
     int (*check_integrity) (modbus_t *ctx, uint8_t *msg,
                             const int msg_length);
+    int (*pre_check_confirmation) (modbus_t *ctx, const uint8_t *req,
+                                   const uint8_t *rsp, int rsp_length);
     int (*connect) (modbus_t *ctx);
     void (*close) (modbus_t *ctx);
     int (*flush) (modbus_t *ctx);
@@ -105,8 +107,8 @@ struct _modbus {
     int s;
     int debug;
     int error_recovery;
-    struct timeval timeout_begin;
-    struct timeval timeout_end;
+    struct timeval response_timeout;
+    struct timeval byte_timeout;
     const modbus_backend_t *backend;
     void *backend_data;
 };
