@@ -1,18 +1,19 @@
 /*
- * Copyright © 2001-2010 Stéphane Raimbault <stephane.raimbault@gmail.com>
+ * Copyright © 2001-2011 Stéphane Raimbault <stephane.raimbault@gmail.com>
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  *
  * This library implements the Modbus protocol.
@@ -22,11 +23,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 #include <errno.h>
 #include <limits.h>
+#include <time.h>
 
 #include <config.h>
 
@@ -98,18 +97,38 @@ void _error_print(modbus_t *ctx, const char *context)
     }
 }
 
+int _sleep_and_flush(modbus_t *ctx)
+{
+#ifdef _WIN32
+    /* usleep doesn't exist on Windows */
+    Sleep((ctx->response_timeout.tv_sec * 1000) +
+          (ctx->response_timeout.tv_usec / 1000));
+#else
+    /* usleep source code */
+    struct timespec request, remaining;
+    request.tv_sec = ctx->response_timeout.tv_sec;
+    request.tv_nsec = ((long int)ctx->response_timeout.tv_usec % 1000000)
+        * 1000;
+    while (nanosleep(&request, &remaining) == -1 && errno == EINTR)
+        request = remaining;
+#endif
+    return modbus_flush(ctx);
+}
+
 int modbus_flush(modbus_t *ctx)
 {
-    return ctx->backend->flush(ctx);
+    int rc = ctx->backend->flush(ctx);
+    if (rc != -1 && ctx->debug) {
+        printf("%d bytes flushed\n", rc);
+    }
+    return rc;
 }
 
 /* Computes the length of the expected response */
 static unsigned int compute_response_length_from_request(modbus_t *ctx, uint8_t *req)
 {
     int length;
-    int offset;
-
-    offset = ctx->backend->header_length;
+    const int offset = ctx->backend->header_length;
 
     switch (req[offset]) {
     case _FC_READ_COILS:
@@ -119,7 +138,7 @@ static unsigned int compute_response_length_from_request(modbus_t *ctx, uint8_t
         length = 2 + (nb / 8) + ((nb % 8) ? 1 : 0);
     }
         break;
-    case _FC_READ_AND_WRITE_REGISTERS:
+    case _FC_WRITE_AND_READ_REGISTERS:
     case _FC_READ_HOLDING_REGISTERS:
     case _FC_READ_INPUT_REGISTERS:
         /* Header + 2 * nb values */
@@ -140,34 +159,41 @@ static unsigned int compute_response_length_from_request(modbus_t *ctx, uint8_t
 }
 
 /* Sends a request/response */
-static int send_msg(modbus_t *ctx, uint8_t *req, int req_length)
+static int send_msg(modbus_t *ctx, uint8_t *msg, int msg_length)
 {
     int rc;
     int i;
 
-    req_length = ctx->backend->send_msg_pre(req, req_length);
+    msg_length = ctx->backend->send_msg_pre(msg, msg_length);
 
     if (ctx->debug) {
-        for (i = 0; i < req_length; i++)
-            printf("[%.2X]", req[i]);
+        for (i = 0; i < msg_length; i++)
+            printf("[%.2X]", msg[i]);
         printf("\n");
     }
 
     /* In recovery mode, the write command will be issued until to be
        successful! Disabled by default. */
     do {
-        rc = ctx->backend->send(ctx, req, req_length);
+        rc = ctx->backend->send(ctx, msg, msg_length);
         if (rc == -1) {
             _error_print(ctx, NULL);
-            if (ctx->error_recovery &&
-                (errno == EBADF || errno == ECONNRESET || errno == EPIPE)) {
-                modbus_close(ctx);
-                modbus_connect(ctx);
+            if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_LINK) {
+                int saved_errno = errno;
+
+                if ((errno == EBADF || errno == ECONNRESET || errno == EPIPE)) {
+                    modbus_close(ctx);
+                    modbus_connect(ctx);
+                } else {
+                    _sleep_and_flush(ctx);
+                }
+                errno = saved_errno;
             }
         }
-    } while (ctx->error_recovery && rc == -1);
+    } while ((ctx->error_recovery & MODBUS_ERROR_RECOVERY_LINK) &&
+             rc == -1);
 
-    if (rc > 0 && rc != req_length) {
+    if (rc > 0 && rc != msg_length) {
         errno = EMBBADDATA;
         return -1;
     }
@@ -175,6 +201,34 @@ static int send_msg(modbus_t *ctx, uint8_t *req, int req_length)
     return rc;
 }
 
+int modbus_send_raw_request(modbus_t *ctx, uint8_t *raw_req, int raw_req_length)
+{
+    sft_t sft;
+    uint8_t req[MAX_MESSAGE_LENGTH];
+    int req_length;
+
+    if (raw_req_length < 2) {
+        /* The raw request must contain function and slave at least */
+        errno = EINVAL;
+        return -1;
+    }
+
+    sft.slave = raw_req[0];
+    sft.function = raw_req[1];
+    /* The t_id is left to zero */
+    sft.t_id = 0;
+    /* This response function only set the header so it's convenient here */
+    req_length = ctx->backend->build_response_basis(&sft, req);
+
+    if (raw_req_length > 2) {
+        /* Copy data after function code */
+        memcpy(req + req_length, raw_req + 2, raw_req_length - 2);
+        req_length += raw_req_length - 2;
+    }
+
+    return send_msg(ctx, req, req_length);
+}
+
 /*
     ---------- Request     Indication ----------
     | Client | ---------------------->| Server |
@@ -200,7 +254,7 @@ static uint8_t compute_meta_length_after_function(int function,
         } else if (function == _FC_WRITE_MULTIPLE_COILS ||
                    function == _FC_WRITE_MULTIPLE_REGISTERS) {
             length = 5;
-        } else if (function == _FC_READ_AND_WRITE_REGISTERS) {
+        } else if (function == _FC_WRITE_AND_READ_REGISTERS) {
             length = 9;
         } else {
             /* _FC_READ_EXCEPTION_STATUS, _FC_REPORT_SLAVE_ID */
@@ -224,7 +278,8 @@ static uint8_t compute_meta_length_after_function(int function,
 }
 
 /* Computes the length to read after the meta information (address, count, etc) */
-static int compute_data_length_after_meta(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
+static int compute_data_length_after_meta(modbus_t *ctx, uint8_t *msg,
+                                          msg_type_t msg_type)
 {
     int function = msg[ctx->backend->header_length];
     int length;
@@ -235,7 +290,7 @@ static int compute_data_length_after_meta(modbus_t *ctx, uint8_t *msg, msg_type_
         case _FC_WRITE_MULTIPLE_REGISTERS:
             length = msg[ctx->backend->header_length + 5];
             break;
-        case _FC_READ_AND_WRITE_REGISTERS:
+        case _FC_WRITE_AND_READ_REGISTERS:
             length = msg[ctx->backend->header_length + 9];
             break;
         default:
@@ -245,7 +300,7 @@ static int compute_data_length_after_meta(modbus_t *ctx, uint8_t *msg, msg_type_
         /* MSG_CONFIRMATION */
         if (function <= _FC_READ_INPUT_REGISTERS ||
             function == _FC_REPORT_SLAVE_ID ||
-            function == _FC_READ_AND_WRITE_REGISTERS) {
+            function == _FC_WRITE_AND_READ_REGISTERS) {
             length = msg[ctx->backend->header_length + 1];
         } else {
             length = 0;
@@ -276,8 +331,8 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
     int rc;
     fd_set rfds;
     struct timeval tv;
+    struct timeval *p_tv;
     int length_to_read;
-    uint8_t *p_msg;
     int msg_length = 0;
     _step_t step;
 
@@ -302,22 +357,32 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
     if (msg_type == MSG_INDICATION) {
         /* Wait for a message, we don't know when the message will be
          * received */
-        /* FIXME Not infinite */
-        tv.tv_sec = 60;
-        tv.tv_usec = 0;
+        p_tv = NULL;
     } else {
-        tv.tv_sec = ctx->timeout_begin.tv_sec;
-        tv.tv_usec = ctx->timeout_begin.tv_usec;
+        tv.tv_sec = ctx->response_timeout.tv_sec;
+        tv.tv_usec = ctx->response_timeout.tv_usec;
+        p_tv = &tv;
     }
 
-    p_msg = msg;
     while (length_to_read != 0) {
-        rc = ctx->backend->select(ctx, &rfds, &tv, length_to_read);
+        rc = ctx->backend->select(ctx, &rfds, p_tv, length_to_read);
         if (rc == -1) {
+            _error_print(ctx, "select");
+            if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_LINK) {
+                int saved_errno = errno;
+
+                if (errno == ETIMEDOUT) {
+                    _sleep_and_flush(ctx);
+                } else if (errno == EBADF) {
+                    modbus_close(ctx);
+                    modbus_connect(ctx);
+                }
+                errno = saved_errno;
+            }
             return -1;
         }
 
-        rc = ctx->backend->recv(ctx, p_msg, length_to_read);
+        rc = ctx->backend->recv(ctx, msg + msg_length, length_to_read);
         if (rc == 0) {
             errno = ECONNRESET;
             rc = -1;
@@ -325,12 +390,14 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
 
         if (rc == -1) {
             _error_print(ctx, "read");
-            if (ctx->error_recovery && (errno == ECONNRESET ||
-                                        errno == ECONNREFUSED)) {
+            if ((ctx->error_recovery & MODBUS_ERROR_RECOVERY_LINK) &&
+                (errno == ECONNRESET || errno == ECONNREFUSED ||
+                 errno == EBADF)) {
+                int saved_errno = errno;
                 modbus_close(ctx);
                 modbus_connect(ctx);
                 /* Could be removed by previous calls */
-                errno = ECONNRESET;
+                errno = saved_errno;
             }
             return -1;
         }
@@ -339,11 +406,9 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
         if (ctx->debug) {
             int i;
             for (i=0; i < rc; i++)
-                printf("<%.2X>", p_msg[i]);
+                printf("<%.2X>", msg[msg_length + i]);
         }
 
-        /* Moves the pointer to receive other data */
-        p_msg = &(p_msg[rc]);
         /* Sums bytes received */
         msg_length += rc;
         /* Computes remaining bytes */
@@ -376,10 +441,12 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
         }
 
         if (length_to_read > 0) {
-            /* If no character at the buffer wait
-               TIME_OUT_END_OF_TRAME before raising an error. */
-            tv.tv_sec = ctx->timeout_end.tv_sec;
-            tv.tv_usec = ctx->timeout_end.tv_usec;
+            /* If there is no character in the buffer, the allowed timeout
+               interval between two consecutive bytes is defined by
+               byte_timeout */
+            tv.tv_sec = ctx->byte_timeout.tv_sec;
+            tv.tv_usec = ctx->byte_timeout.tv_usec;
+            p_tv = &tv;
         }
     }
 
@@ -389,47 +456,50 @@ static int receive_msg(modbus_t *ctx, uint8_t *msg, msg_type_t msg_type)
     return ctx->backend->check_integrity(ctx, msg, msg_length);
 }
 
-/* Receive the request from a modbus master, requires the socket file descriptor
-   etablished with the master device in argument or -1 to use the internal one
-   of modbus_t.
-
-   The function shall return the request received and its byte length if
-   successul. Otherwise, it shall return -1 and errno is set. */
-int modbus_receive(modbus_t *ctx, int sockfd, uint8_t *req)
+/* Receive the request from a modbus master */
+int modbus_receive(modbus_t *ctx, uint8_t *req)
 {
-    if (sockfd != -1) {
-        ctx->s = sockfd;
-    }
-
     return receive_msg(ctx, req, MSG_INDICATION);
 }
 
-/* Receives the response and checks values.
+/* Receives the confirmation.
 
-   The function shall return the number of values (bits or words) and the
-   response if successful. Otherwise, its shall return -1 and errno is set.
+   The function shall store the read response in rsp and return the number of
+   values (bits or words). Otherwise, its shall return -1 and errno is set.
 
-   Note: all functions used to send or receive data with modbus return
-   these values. */
-static int receive_msg_req(modbus_t *ctx, uint8_t *req, uint8_t *rsp)
+   The function doesn't check the confirmation is the expected response to the
+   initial request.
+*/
+int modbus_receive_confirmation(modbus_t *ctx, uint8_t *rsp)
+{
+    return receive_msg(ctx, rsp, MSG_CONFIRMATION);
+}
+
+static int check_confirmation(modbus_t *ctx, uint8_t *req,
+                              uint8_t *rsp, int rsp_length)
 {
     int rc;
     int rsp_length_computed;
-    int offset = ctx->backend->header_length;
+    const int offset = ctx->backend->header_length;
 
-    rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
-    if (rc == -1) {
-        return -1;
+    if (ctx->backend->pre_check_confirmation) {
+        rc = ctx->backend->pre_check_confirmation(ctx, req, rsp, rsp_length);
+        if (rc == -1) {
+            if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_PROTOCOL) {
+                _sleep_and_flush(ctx);
+            }
+            return -1;
+        }
     }
 
     rsp_length_computed = compute_response_length_from_request(ctx, req);
 
     /* Check length */
-    if (rc == rsp_length_computed ||
+    if (rsp_length == rsp_length_computed ||
         rsp_length_computed == MSG_LENGTH_UNDEFINED) {
         int req_nb_value;
         int rsp_nb_value;
-        int function = rsp[offset];
+        const int function = rsp[offset];
 
         /* Check function code */
         if (function != req[offset]) {
@@ -438,6 +508,9 @@ static int receive_msg_req(modbus_t *ctx, uint8_t *req, uint8_t *rsp)
                         "Received function not corresponding to the request (%d != %d)\n",
                         function, req[offset]);
             }
+            if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_PROTOCOL) {
+                _sleep_and_flush(ctx);
+            }
             errno = EMBBADDATA;
             return -1;
         }
@@ -453,7 +526,7 @@ static int receive_msg_req(modbus_t *ctx, uint8_t *req, uint8_t *rsp)
             req_nb_value = (req_nb_value / 8) + ((req_nb_value % 8) ? 1 : 0);
             rsp_nb_value = rsp[offset + 1];
             break;
-        case _FC_READ_AND_WRITE_REGISTERS:
+        case _FC_WRITE_AND_READ_REGISTERS:
         case _FC_READ_HOLDING_REGISTERS:
         case _FC_READ_INPUT_REGISTERS:
             /* Read functions 1 value = 2 bytes */
@@ -483,10 +556,15 @@ static int receive_msg_req(modbus_t *ctx, uint8_t *req, uint8_t *rsp)
                         "Quantity not corresponding to the request (%d != %d)\n",
                         rsp_nb_value, req_nb_value);
             }
+
+            if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_PROTOCOL) {
+                _sleep_and_flush(ctx);
+            }
+
             errno = EMBBADDATA;
             rc = -1;
         }
-    } else if (rc == (offset + 2 + ctx->backend->checksum_length) &&
+    } else if (rsp_length == (offset + 2 + ctx->backend->checksum_length) &&
                req[offset] == (rsp[offset] - 0x80)) {
         /* EXCEPTION CODE RECEIVED */
 
@@ -502,7 +580,10 @@ static int receive_msg_req(modbus_t *ctx, uint8_t *req, uint8_t *rsp)
         if (ctx->debug) {
             fprintf(stderr,
                     "Message length not corresponding to the computed length (%d != %d)\n",
-                    rc, rsp_length_computed);
+                    rsp_length, rsp_length_computed);
+        }
+        if (ctx->error_recovery & MODBUS_ERROR_RECOVERY_PROTOCOL) {
+            _sleep_and_flush(ctx);
         }
         errno = EMBBADDATA;
         rc = -1;
@@ -551,7 +632,7 @@ static int response_exception(modbus_t *ctx, sft_t *sft,
     return rsp_length;
 }
 
-/* Send a response to the receive request.
+/* Send a response to the received request.
    Analyses the request and constructs a response.
 
    If an error occurs, this function construct the response
@@ -761,13 +842,22 @@ int modbus_reply(modbus_t *ctx, const uint8_t *req,
         }
     }
         break;
-    case _FC_REPORT_SLAVE_ID:
+    case _FC_REPORT_SLAVE_ID: {
+        int str_len;
+        int byte_count_pos;
+
         rsp_length = ctx->backend->build_response_basis(&sft, rsp);
-        /* 2 bytes */
-        rsp[rsp_length++] = 2;
-        rsp[rsp_length++] = ctx->slave;
-        /* Slave is ON */
+        /* Skip byte count for now */
+        byte_count_pos = rsp_length++;
+        rsp[rsp_length++] = _REPORT_SLAVE_ID;
+        /* Run indicator status to ON */
         rsp[rsp_length++] = 0xFF;
+        /* LMB + length of LIBMODBUS_VERSION_STRING */
+        str_len = 3 + strlen(LIBMODBUS_VERSION_STRING);
+        memcpy(rsp + rsp_length, "LMB" LIBMODBUS_VERSION_STRING, str_len);
+        rsp_length += str_len;
+        rsp[byte_count_pos] = rsp_length - byte_count_pos - 1;
+    }
         break;
     case _FC_READ_EXCEPTION_STATUS:
         if (ctx->debug) {
@@ -777,7 +867,7 @@ int modbus_reply(modbus_t *ctx, const uint8_t *req,
         return -1;
         break;
 
-    case _FC_READ_AND_WRITE_REGISTERS: {
+    case _FC_WRITE_AND_READ_REGISTERS: {
         int nb = (req[offset + 3] << 8) + req[offset + 4];
         uint16_t address_write = (req[offset + 5] << 8) + req[offset + 6];
         int nb_write = (req[offset + 7] << 8) + req[offset + 8];
@@ -786,7 +876,7 @@ int modbus_reply(modbus_t *ctx, const uint8_t *req,
             (address_write + nb_write) > mb_mapping->nb_registers) {
             if (ctx->debug) {
                 fprintf(stderr,
-                        "Illegal data read address %0X or write address %0X in read_and_write_registers\n",
+                        "Illegal data read address %0X or write address %0X write_and_read_registers\n",
                         address + nb, address_write + nb_write);
             }
             rsp_length = response_exception(ctx, &sft,
@@ -855,7 +945,7 @@ int modbus_reply_exception(modbus_t *ctx, const uint8_t *req,
 
 /* Reads IO status */
 static int read_io_status(modbus_t *ctx, int function,
-                          int addr, int nb, uint8_t *data_dest)
+                          int addr, int nb, uint8_t *dest)
 {
     int rc;
     int req_length;
@@ -872,7 +962,11 @@ static int read_io_status(modbus_t *ctx, int function,
         int offset;
         int offset_end;
 
-        rc = receive_msg_req(ctx, req, rsp);
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
         if (rc == -1)
             return -1;
 
@@ -883,7 +977,7 @@ static int read_io_status(modbus_t *ctx, int function,
             temp = rsp[i];
 
             for (bit = 0x01; (bit & 0xff) && (pos < nb);) {
-                data_dest[pos++] = (temp & bit) ? TRUE : FALSE;
+                dest[pos++] = (temp & bit) ? TRUE : FALSE;
                 bit = bit << 1;
             }
 
@@ -895,7 +989,7 @@ static int read_io_status(modbus_t *ctx, int function,
 
 /* Reads the boolean status of bits and sets the array elements
    in the destination to TRUE or FALSE (single bits). */
-int modbus_read_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
+int modbus_read_bits(modbus_t *ctx, int addr, int nb, uint8_t *dest)
 {
     int rc;
 
@@ -909,7 +1003,7 @@ int modbus_read_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
         return -1;
     }
 
-    rc = read_io_status(ctx, _FC_READ_COILS, addr, nb, data_dest);
+    rc = read_io_status(ctx, _FC_READ_COILS, addr, nb, dest);
 
     if (rc == -1)
         return -1;
@@ -919,7 +1013,7 @@ int modbus_read_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
 
 
 /* Same as modbus_read_bits but reads the remote device input table */
-int modbus_read_input_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
+int modbus_read_input_bits(modbus_t *ctx, int addr, int nb, uint8_t *dest)
 {
     int rc;
 
@@ -933,7 +1027,7 @@ int modbus_read_input_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
         return -1;
     }
 
-    rc = read_io_status(ctx, _FC_READ_DISCRETE_INPUTS, addr, nb, data_dest);
+    rc = read_io_status(ctx, _FC_READ_DISCRETE_INPUTS, addr, nb, dest);
 
     if (rc == -1)
         return -1;
@@ -943,7 +1037,7 @@ int modbus_read_input_bits(modbus_t *ctx, int addr, int nb, uint8_t *data_dest)
 
 /* Reads the data from a remove device and put that data into an array */
 static int read_registers(modbus_t *ctx, int function, int addr, int nb,
-                          uint16_t *data_dest)
+                          uint16_t *dest)
 {
     int rc;
     int req_length;
@@ -967,16 +1061,19 @@ static int read_registers(modbus_t *ctx, int function, int addr, int nb,
         int offset;
         int i;
 
-        rc = receive_msg_req(ctx, req, rsp);
-        if (rc == -1) {
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
+        if (rc == -1)
             return -1;
-        }
 
         offset = ctx->backend->header_length;
 
         for (i = 0; i < rc; i++) {
             /* shift reg hi_byte to temp OR with lo_byte */
-            data_dest[i] = (rsp[offset + 2 + (i << 1)] << 8) |
+            dest[i] = (rsp[offset + 2 + (i << 1)] << 8) |
                 rsp[offset + 3 + (i << 1)];
         }
     }
@@ -986,7 +1083,7 @@ static int read_registers(modbus_t *ctx, int function, int addr, int nb,
 
 /* Reads the holding registers of remote device and put the data into an
    array */
-int modbus_read_registers(modbus_t *ctx, int addr, int nb, uint16_t *data_dest)
+int modbus_read_registers(modbus_t *ctx, int addr, int nb, uint16_t *dest)
 {
     int status;
 
@@ -1001,13 +1098,13 @@ int modbus_read_registers(modbus_t *ctx, int addr, int nb, uint16_t *data_dest)
     }
 
     status = read_registers(ctx, _FC_READ_HOLDING_REGISTERS,
-                            addr, nb, data_dest);
+                            addr, nb, dest);
     return status;
 }
 
 /* Reads the input registers of remote device and put the data into an array */
 int modbus_read_input_registers(modbus_t *ctx, int addr, int nb,
-                                uint16_t *data_dest)
+                                uint16_t *dest)
 {
     int status;
 
@@ -1020,7 +1117,7 @@ int modbus_read_input_registers(modbus_t *ctx, int addr, int nb,
     }
 
     status = read_registers(ctx, _FC_READ_INPUT_REGISTERS,
-                            addr, nb, data_dest);
+                            addr, nb, dest);
 
     return status;
 }
@@ -1039,37 +1136,32 @@ static int write_single(modbus_t *ctx, int function, int addr, int value)
     if (rc > 0) {
         /* Used by write_bit and write_register */
         uint8_t rsp[_MIN_REQ_LENGTH];
-        rc = receive_msg_req(ctx, req, rsp);
+
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
     }
 
     return rc;
 }
 
 /* Turns ON or OFF a single bit of the remote device */
-int modbus_write_bit(modbus_t *ctx, int addr, int state)
+int modbus_write_bit(modbus_t *ctx, int addr, int status)
 {
-    int status;
-
-    if (state)
-        state = 0xFF00;
-
-    status = write_single(ctx, _FC_WRITE_SINGLE_COIL, addr, state);
-
-    return status;
+    return write_single(ctx, _FC_WRITE_SINGLE_COIL, addr,
+                        status ? 0xFF00 : 0);
 }
 
 /* Writes a value in one register of the remote device */
 int modbus_write_register(modbus_t *ctx, int addr, int value)
 {
-    int status;
-
-    status = write_single(ctx, _FC_WRITE_SINGLE_REGISTER, addr, value);
-
-    return status;
+    return write_single(ctx, _FC_WRITE_SINGLE_REGISTER, addr, value);
 }
 
 /* Write the bits of the array in the remote device */
-int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *data_src)
+int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *src)
 {
     int rc;
     int i;
@@ -1089,7 +1181,8 @@ int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *data_src)
         return -1;
     }
 
-    req_length = ctx->backend->build_request_basis(ctx, _FC_WRITE_MULTIPLE_COILS,
+    req_length = ctx->backend->build_request_basis(ctx,
+                                                   _FC_WRITE_MULTIPLE_COILS,
                                                    addr, nb, req);
     byte_count = (nb / 8) + ((nb % 8) ? 1 : 0);
     req[req_length++] = byte_count;
@@ -1101,7 +1194,7 @@ int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *data_src)
         req[req_length] = 0;
 
         while ((bit & 0xFF) && (bit_check++ < nb)) {
-            if (data_src[pos++])
+            if (src[pos++])
                 req[req_length] |= bit;
             else
                 req[req_length] &=~ bit;
@@ -1114,7 +1207,12 @@ int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *data_src)
     rc = send_msg(ctx, req, req_length);
     if (rc > 0) {
         uint8_t rsp[MAX_MESSAGE_LENGTH];
-        rc = receive_msg_req(ctx, req, rsp);
+
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
     }
 
 
@@ -1122,7 +1220,7 @@ int modbus_write_bits(modbus_t *ctx, int addr, int nb, const uint8_t *data_src)
 }
 
 /* Write the values from the array to the registers of the remote device */
-int modbus_write_registers(modbus_t *ctx, int addr, int nb, const uint16_t *data_src)
+int modbus_write_registers(modbus_t *ctx, int addr, int nb, const uint16_t *src)
 {
     int rc;
     int i;
@@ -1148,24 +1246,30 @@ int modbus_write_registers(modbus_t *ctx, int addr, int nb, const uint16_t *data
     req[req_length++] = byte_count;
 
     for (i = 0; i < nb; i++) {
-        req[req_length++] = data_src[i] >> 8;
-        req[req_length++] = data_src[i] & 0x00FF;
+        req[req_length++] = src[i] >> 8;
+        req[req_length++] = src[i] & 0x00FF;
     }
 
     rc = send_msg(ctx, req, req_length);
     if (rc > 0) {
         uint8_t rsp[MAX_MESSAGE_LENGTH];
-        rc = receive_msg_req(ctx, req, rsp);
+
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
     }
 
     return rc;
 }
 
-/* Read multiple registers from remote device to dest array and write multiple
-   registers to remote device from data array. */
-int modbus_read_and_write_registers(modbus_t *ctx,
-                                    int read_addr, int read_nb, uint16_t *dest,
-                                    int write_addr, int write_nb, const uint16_t *data)
+/* Write multiple registers from src array to remote device and read multiple
+   registers from remote device to dest array. */
+int modbus_write_and_read_registers(modbus_t *ctx,
+                                    int write_addr, int write_nb, const uint16_t *src,
+                                    int read_addr, int read_nb, uint16_t *dest)
+
 {
     int rc;
     int req_length;
@@ -1174,16 +1278,6 @@ int modbus_read_and_write_registers(modbus_t *ctx,
     uint8_t req[MAX_MESSAGE_LENGTH];
     uint8_t rsp[MAX_MESSAGE_LENGTH];
 
-    if (read_nb > MODBUS_MAX_READ_REGISTERS) {
-        if (ctx->debug) {
-            fprintf(stderr,
-                    "ERROR Too many registers requested (%d > %d)\n",
-                    read_nb, MODBUS_MAX_READ_REGISTERS);
-        }
-        errno = EMBMDATA;
-        return -1;
-    }
-
     if (write_nb > MODBUS_MAX_RW_WRITE_REGISTERS) {
         if (ctx->debug) {
             fprintf(stderr,
@@ -1193,8 +1287,18 @@ int modbus_read_and_write_registers(modbus_t *ctx,
         errno = EMBMDATA;
         return -1;
     }
+
+    if (read_nb > MODBUS_MAX_READ_REGISTERS) {
+        if (ctx->debug) {
+            fprintf(stderr,
+                    "ERROR Too many registers requested (%d > %d)\n",
+                    read_nb, MODBUS_MAX_READ_REGISTERS);
+        }
+        errno = EMBMDATA;
+        return -1;
+    }
     req_length = ctx->backend->build_request_basis(ctx,
-                                                   _FC_READ_AND_WRITE_REGISTERS,
+                                                   _FC_WRITE_AND_READ_REGISTERS,
                                                    read_addr, read_nb, req);
 
     req[req_length++] = write_addr >> 8;
@@ -1205,15 +1309,22 @@ int modbus_read_and_write_registers(modbus_t *ctx,
     req[req_length++] = byte_count;
 
     for (i = 0; i < write_nb; i++) {
-        req[req_length++] = data[i] >> 8;
-        req[req_length++] = data[i] & 0x00FF;
+        req[req_length++] = src[i] >> 8;
+        req[req_length++] = src[i] & 0x00FF;
     }
 
     rc = send_msg(ctx, req, req_length);
     if (rc > 0) {
         int offset;
 
-        rc = receive_msg_req(ctx, req, rsp);
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
+        if (rc == -1)
+            return -1;
+
         offset = ctx->backend->header_length;
 
         /* If rc is negative, the loop is jumped ! */
@@ -1229,7 +1340,7 @@ int modbus_read_and_write_registers(modbus_t *ctx,
 
 /* Send a request to get the slave ID of the device (only available in serial
    communication). */
-int modbus_report_slave_id(modbus_t *ctx, uint8_t *data_dest)
+int modbus_report_slave_id(modbus_t *ctx, uint8_t *dest)
 {
     int rc;
     int req_length;
@@ -1247,16 +1358,20 @@ int modbus_report_slave_id(modbus_t *ctx, uint8_t *data_dest)
         int offset;
         uint8_t rsp[MAX_MESSAGE_LENGTH];
 
-        /* Byte count, slave id, run indicator status,
-           additional data */
-        rc = receive_msg_req(ctx, req, rsp);
+        rc = receive_msg(ctx, rsp, MSG_CONFIRMATION);
+        if (rc == -1)
+            return -1;
+
+        rc = check_confirmation(ctx, req, rsp, rc);
         if (rc == -1)
             return -1;
 
         offset = ctx->backend->header_length + 2;
 
+        /* Byte count, slave id, run indicator status,
+           additional data */
         for (i=0; i < rc; i++) {
-            data_dest[i] = rsp[offset + i];
+            dest[i] = rsp[offset + i];
         }
     }
 
@@ -1270,13 +1385,13 @@ void _modbus_init_common(modbus_t *ctx)
     ctx->s = -1;
 
     ctx->debug = FALSE;
-    ctx->error_recovery = FALSE;
+    ctx->error_recovery = MODBUS_ERROR_RECOVERY_NONE;
 
-    ctx->timeout_begin.tv_sec = 0;
-    ctx->timeout_begin.tv_usec = _TIME_OUT_BEGIN_OF_TRAME;
+    ctx->response_timeout.tv_sec = 0;
+    ctx->response_timeout.tv_usec = _RESPONSE_TIMEOUT;
 
-    ctx->timeout_end.tv_sec = 0;
-    ctx->timeout_end.tv_usec = _TIME_OUT_END_OF_TRAME;
+    ctx->byte_timeout.tv_sec = 0;
+    ctx->byte_timeout.tv_usec = _BYTE_TIMEOUT;
 }
 
 /* Define the slave number */
@@ -1285,23 +1400,11 @@ int modbus_set_slave(modbus_t *ctx, int slave)
     return ctx->backend->set_slave(ctx, slave);
 }
 
-/*
-  When disabled (default), it is expected that the application will check for
-  error returns and deal with them as necessary.
-
-  It's not recommanded to enable error recovery for slave/server.
-
-  When enabled, the library will attempt an immediate reconnection which may
-  hang for several seconds if the network to the remote target unit is down.
-  The write will try a infinite close/connect loop until to be successful and
-  the select/read calls will just try to retablish the connection one time then
-  will return an error (if the connecton was down, the values to read are
-  certainly not available anymore after reconnection, except for slave/server).
-*/
-int modbus_set_error_recovery(modbus_t *ctx, int enabled)
+int modbus_set_error_recovery(modbus_t *ctx,
+                              modbus_error_recovery_mode error_recovery)
 {
-    if (enabled == TRUE || enabled == FALSE) {
-        ctx->error_recovery = (uint8_t) enabled;
+    if (error_recovery >= 0) {
+        ctx->error_recovery = (uint8_t) error_recovery;
     } else {
         errno = EINVAL;
         return -1;
@@ -1310,28 +1413,36 @@ int modbus_set_error_recovery(modbus_t *ctx, int enabled)
     return 0;
 }
 
-/* Get the timeout of begin of trame */
-void modbus_get_timeout_begin(modbus_t *ctx, struct timeval *timeout)
+void modbus_set_socket(modbus_t *ctx, int socket)
 {
-    *timeout = ctx->timeout_begin;
+    ctx->s = socket;
 }
 
-/* Set timeout when waiting the beginning of a trame */
-void modbus_set_timeout_begin(modbus_t *ctx, const struct timeval *timeout)
+int modbus_get_socket(modbus_t *ctx)
 {
-    ctx->timeout_begin = *timeout;
+    return ctx->s;
 }
 
-/* Get the timeout of end of trame */
-void modbus_get_timeout_end(modbus_t *ctx, struct timeval *timeout)
+/* Get the timeout interval used to wait for a response */
+void modbus_get_response_timeout(modbus_t *ctx, struct timeval *timeout)
 {
-    *timeout = ctx->timeout_end;
+    *timeout = ctx->response_timeout;
 }
 
-/* Set timeout when waiting the end of a trame */
-void modbus_set_timeout_end(modbus_t *ctx, const struct timeval *timeout)
+void modbus_set_response_timeout(modbus_t *ctx, const struct timeval *timeout)
 {
-    ctx->timeout_end = *timeout;
+    ctx->response_timeout = *timeout;
+}
+
+/* Get the timeout interval between two consecutive bytes of a message */
+void modbus_get_byte_timeout(modbus_t *ctx, struct timeval *timeout)
+{
+    *timeout = ctx->byte_timeout;
+}
+
+void modbus_set_byte_timeout(modbus_t *ctx, const struct timeval *timeout)
+{
+    ctx->byte_timeout = *timeout;
 }
 
 int modbus_get_header_length(modbus_t *ctx)
@@ -1344,7 +1455,6 @@ int modbus_connect(modbus_t *ctx)
     return ctx->backend->connect(ctx);
 }
 
-/* Closes a  connection */
 void modbus_close(modbus_t *ctx)
 {
     if (ctx == NULL)
@@ -1353,7 +1463,6 @@ void modbus_close(modbus_t *ctx)
     ctx->backend->close(ctx);
 }
 
-/* Free an initialized modbus_t */
 void modbus_free(modbus_t *ctx)
 {
     if (ctx == NULL)
@@ -1363,7 +1472,6 @@ void modbus_free(modbus_t *ctx)
     free(ctx);
 }
 
-/* Activates the debug messages */
 void modbus_set_debug(modbus_t *ctx, int boolean)
 {
     ctx->debug = boolean;
